"""
    Support instabot's methods.
"""

from __future__ import unicode_literals

import codecs
import os
import re
import sys

import huepy


def check_if_file_exists(file_path, quiet=False):
    if not os.path.exists(file_path):
        if not quiet:
            print("Can't find '%s' file." % file_path)
        return False
    return True


def read_list_from_file(file_path, quiet=False):
    """
        Reads list from file. One line - one item.
        Returns the list if file items.
    """
    try:
        if not check_if_file_exists(file_path, quiet=quiet):
            return []
        with codecs.open(file_path, "r", encoding="utf-8") as f:
            content = f.readlines()
            if sys.version_info[0] < 3:
                content = [str(item.encode('utf8')) for item in content]
            content = [item.strip() for item in content]
            return [i for i in content if i]
    except Exception as exception:
        print(str(exception))
        return []


def console_print(self, text, color=None):
    if self.verbosity:
        text = '\n' + text
        if color is not None:
            text = getattr(huepy, color)(text)
        print(text)


def extract_urls(text):
    url_regex = r'((?:(?:http|https|Http|Https|rtsp|Rtsp)://(?:(?:[a-zA-Z0-9$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:%[a-fA-F0-9]{2})){1,64}(?::(?:[a-zA-Z0-9$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:%[a-fA-F0-9]{2})){1,25})?@)?)?(?:(?:(?:[a-zA-Z0-9\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF\_][a-zA-Z0-9\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF\_\-]{0,64}\.)+(?:(?:aero|arpa|asia|a[cdefgilmnoqrstuwxz])|(?:biz|b[abdefghijmnorstvwyz])|(?:cat|com|coop|c[acdfghiklmnoruvxyz])|d[ejkmoz]|(?:edu|e[cegrstu])|f[ijkmor]|(?:gov|g[abdefghilmnpqrstuwy])|h[kmnrtu]|(?:info|int|i[delmnoqrst])|(?:jobs|j[emop])|k[eghimnprwyz]|l[abcikrstuvy]|(?:mil|mobi|museum|m[acdeghklmnopqrstuvwxyz])|(?:name|net|n[acefgilopruz])|(?:org|om)|(?:pro|p[aefghklmnrstwy])|qa|r[eosuw]|s[abcdeghijklmnortuvyz]|(?:tel|travel|t[cdfghjklmnoprtvwz])|u[agksyz]|v[aceginu]|w[fs]|(?:\u03B4\u03BF\u03BA\u03B9\u03BC\u03AE|\u0438\u0441\u043F\u044B\u0442\u0430\u043D\u0438\u0435|\u0440\u0444|\u0441\u0440\u0431|\u05D8\u05E2\u05E1\u05D8|\u0622\u0632\u0645\u0627\u06CC\u0634\u06CC|\u0625\u062E\u062A\u0628\u0627\u0631|\u0627\u0644\u0627\u0631\u062F\u0646|\u0627\u0644\u062C\u0632\u0627\u0626\u0631|\u0627\u0644\u0633\u0639\u0648\u062F\u064A\u0629|\u0627\u0644\u0645\u063A\u0631\u0628|\u0627\u0645\u0627\u0631\u0627\u062A|\u0628\u06BE\u0627\u0631\u062A|\u062A\u0648\u0646\u0633|\u0633\u0648\u0631\u064A\u0629|\u0641\u0644\u0633\u0637\u064A\u0646|\u0642\u0637\u0631|\u0645\u0635\u0631|\u092A\u0930\u0940\u0915\u094D\u0937\u093E|\u092D\u093E\u0930\u0924|\u09AD\u09BE\u09B0\u09A4|\u0A2D\u0A3E\u0A30\u0A24|\u0AAD\u0ABE\u0AB0\u0AA4|\u0B87\u0BA8\u0BCD\u0BA4\u0BBF\u0BAF\u0BBE|\u0B87\u0BB2\u0B99\u0BCD\u0B95\u0BC8|\u0B9A\u0BBF\u0B99\u0BCD\u0B95\u0BAA\u0BCD\u0BAA\u0BC2\u0BB0\u0BCD|\u0BAA\u0BB0\u0BBF\u0B9F\u0BCD\u0B9A\u0BC8|\u0C2D\u0C3E\u0C30\u0C24\u0C4D|\u0DBD\u0D82\u0D9A\u0DCF|\u0E44\u0E17\u0E22|\u30C6\u30B9\u30C8|\u4E2D\u56FD|\u4E2D\u570B|\u53F0\u6E7E|\u53F0\u7063|\u65B0\u52A0\u5761|\u6D4B\u8BD5|\u6E2C\u8A66|\u9999\u6E2F|\uD14C\uC2A4\uD2B8|\uD55C\uAD6D|xn--0zwm56d|xn--11b5bs3a9aj6g|xn--3e0b707e|xn--45brj9c|xn--80akhbyknj4f|xn--90a3ac|xn--9t4b11yi5a|xn--clchc0ea0b2g2a9gcd|xn--deba0ad|xn--fiqs8s|xn--fiqz9s|xn--fpcrj9c3d|xn--fzc2c9e2c|xn--g6w251d|xn--gecrj9c|xn--h2brj9c|xn--hgbk6aj7f53bba|xn--hlcj6aya9esc7a|xn--j6w193g|xn--jxalpdlp|xn--kgbechtv|xn--kprw13d|xn--kpry57d|xn--lgbbat1ad8j|xn--mgbaam7a8h|xn--mgbayh7gpa|xn--mgbbh1a71e|xn--mgbc0a9azcg|xn--mgberp4a5d4ar|xn--o3cw4h|xn--ogbpf8fl|xn--p1ai|xn--pgbs0dh|xn--s9brj9c|xn--wgbh1c|xn--wgbl6a|xn--xkc2al3hye2a|xn--xkc2dl3a5ee0h|xn--yfro4i67o|xn--ygbi2ammx|xn--zckzah|xxx)|y[et]|z[amw]))|(?:(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9])\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[0-9])))(?::\d{1,5})?(?:/(?:(?:[a-zA-Z0-9\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF\;\/\?\:\@\&\=\#\~\-\.\+\!\*\'\(\)\,\_])|(?:%[a-fA-F0-9]{2}))*)?)(?:\b|$)'
    urls = re.findall(url_regex, text)

    return urls



DEFAULT_RESPONSE = {'status': 'ok'}

TEST_CAPTION_ITEM = {
    "bit_flags": 0,
    "content_type": "comment",
    "created_at": 1494733796,
    "created_at_utc": 1494733796,
    "did_report_as_spam": False,
    "pk": 17856098620165444,
    "status": "Active",
    "text": "Old Harry Rocks, Dorset, UK\n\n#oldharryrocks #dorset #uk #rocks #clouds #water #photoshoot #nature #amazing #beautifulsky #sky #landscape #nice #beautiful #awesome #landscapes #l4l #f4f",
    "type": 1,
    "user": {
            "full_name": "The best earth places",
            "is_private": False,
            "is_verified": False,
            "pk": 182696006,
            "profile_pic_id": "1477989239094674784_182696006",
            "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/703a0877fc8653d8b1ec17d57cf0aed6/5B2669EF/t51.2885-19/s150x150/17332952_229750480834181_3303899473574363136_a.jpg",
            "username": "best.earth.places"
    },
    "user_id": 182696006
}

TEST_COMMENT_ITEM = {
    "bit_flags": 0,
    "comment_like_count": 1,
    "content_type": "comment",
    "created_at": 1494751960,
    "created_at_utc": 1494751960,
    "did_report_as_spam": False,
    "has_liked_comment": True,
    "inline_composer_display_condition": "never",
    "pk": 17856583722163490,
    "status": "Active",
    "text": "Wow awesome take!",
    "type": 0,
    "user": {
        "full_name": "Jon",
        "is_private": False,
        "is_verified": False,
        "pk": 4236956175,
        "profile_pic_id": "1699734799357160789_42369123453",
        "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/cf320d98994fasdasdas5fa53c698996bfa/5B0AD6BE/t51.2885-19/s150x150/26398129_141825939841889_6195199731287719936_n.jpg",
        "username": "test_user_name"
    },
    "user_id": 4236736455
}

TEST_USER_ITEM = {
    "pk": 1234543212321,
    "username": "test_username",
    "full_name": "Test Username",
    "is_private": False,
    "profile_pic_url": "https://scontent-arn2-1.cdninstagram.com/vp/6f3e8913b4e9c3153bc15669c47c9519/5AE4F003/t51.2885-19/s150x150/21827786_319483561795594_805639369123123123_n.jpg",
    "profile_pic_id": "120334920291626198135_614512764",
    "friendship_status": {
                "following": True,
                "is_private": False,
                "incoming_request": False,
                "outgoing_request": False,
                "is_bestie": False
    },
    "is_verified": False,
    "has_anonymous_profile_picture": False,
    "follower_count": 470,
    "byline": "470 followers",
    "social_context": "Following",
    "search_social_context": "Following",
    "mutual_followers_count": 0.0,
    "unseen_count": 0
}

TEST_USERNAME_INFO_ITEM = {
    "aggregate_promote_engagement": False,
    "allowed_commenter_type": "any",
    "auto_expand_chaining": False,
    "besties_count": 0,
    "biography": "TEST",
    "can_be_tagged_as_sponsor": False,
    "can_boost_post": False,
    "can_convert_to_business": True,
    "can_create_sponsor_tags": False,
    "can_follow_hashtag": False,
    "can_see_organic_insights": False,
    "external_url": "",
    "feed_post_reshare_disabled": False,
    "follower_count": 100,
    "following_count": 100,
    "full_name": "Test user",
    "has_anonymous_profile_picture": False,
    "has_chaining": True,
    "has_highlight_reels": False,
    "has_profile_video_feed": False,
    "hd_profile_pic_url_info": {
        "height": 1080,
        "url": "https://scontent-ams3-1.cdninstagram.com/vp/b99141f9080640b7bec84e7a67caf69e/5B27B215/t51.7777-19/17332952_229750480834181_377777777357477136_a.jpg",
        "width": 1080
    },
    "hd_profile_pic_versions": [
        {
            "height": 320,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/d8083f7cb9ec5e0c265ed832867ec62f/5B3C2B1F/t51.7777-19/s320x320/17332952_229750487777777_330377473574363136_a.jpg",
            "width": 320
        },
        {
            "height": 640,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/12596165258c466f05979c1be6e95b07/5B294470/t51.7777-19/s640x640/17332952_229750487777777_330377473574363136_a.jpg",
            "width": 640
        }
    ],
    "include_direct_blacklist_status": True,
    "is_business": False,
    "is_needy": False,
    "is_private": False,
    "is_profile_action_needed": False,
    "is_verified": False,
    "is_video_creator": False,
    "media_count": 100,
    "pk": 7777777777,
    "profile_pic_id": "14779892390947777777_18277776",
    "profile_pic_url": "https://scontent-ams3-1.cdninstagram.com/vp/d056984e148d566bbb15f7eaad55304e/5B4DF6EF/t51.7777-19/s150x150/17377752_27777770480834181_3303899473577777736_a.jpg",
    "recently_bestied_by_count": 0,
    "reel_auto_archive": "unset",
    "show_besties_badge": True,
    "show_business_conversion_icon": False,
    "show_conversion_edit_entry": True,
    "show_insights_terms": False,
    "username": "test_user",
    "usertag_review_enabled": False,
    "usertags_count": 16
}

TEST_SEARCH_USERNAME_ITEM = {
    "biography": "Some biography",
    "external_url": "",
    "follower_count": 100,
    "following_count": 100,
    "full_name": "Awesome Test User",
    "has_anonymous_profile_picture": False,
    "has_chaining": False,
    "hd_profile_pic_url_info": {
        "height": 440,
        "url": "https://scontent-ams3-1.cdninstagram.com/vp/0049430734318b5063c4cb2cee26862d/5B399C46/t51.2485-29/28435967_574234472958498_24213573932476253432_n.jpg",
        "width": 440
    },
    "hd_profile_pic_versions": [
        {
            "height": 320,
            "url": "https://scontent-ams3-1.cdninstagram.com/vp/9a869a4fffb0f65b4d850cae433ab00c/5B3B894C/t55.25585-19/s320x320/28435967_574234472958498_24213573958234234236_n.jpg",
            "width": 320
        }
    ],
    "is_business": False,
    "is_favorite": False,
    "is_private": False,
    "is_verified": False,
    "media_count": 100,
    "pk": 777777777,
    "profile_pic_id": "1727616424669463993_721567777777",
    "profile_pic_url": "https://scontent-ams3-1.cdninstagram.com/vp/86a59b5dadc88e6b6a723b73a6ba25a7/5B271FBC/t51.7777-19/s150x150/28435967_574234472958498_24213577777777777_n.jpg",
    "reel_auto_archive": "unset",
    "username": "awesome.test.user",
    "usertags_count": 0
}

TEST_PHOTO_ITEM = {
    'taken_at': 1281669687,
    'pk': 1234,
    'id': '1234_19',
    'device_timestamp': 1281669538,
    'media_type': 1,
    'code': 'TS',
    'client_cache_key': 'MTIzNA==.2',
    'filter_type': 0,
    'image_versions2': {
        'candidates': [
            {
                'width': 612,
                'height': 612,
                'url': 'https://scontent-amt2-1.cdninstagram.com/vp/9ef94dbfea2b8cdb2ba5c9b45f1945fd/5AEFE328/t51.2885-15/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2'
            },
            {
                'width': 240,
                'height': 240,
                'url': 'https://scontent-amt2-1.cdninstagram.com/vp/3f4554f2e45bf356951b2a437ab7d5b1/5ADE3631/t51.2885-15/s240x240/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2'}]
    },
    'original_width': 612,
    'original_height': 612,
    'lat': 37.3988349713,
    'lng': -122.027721405,
    'user': {
        'pk': 19,
        'username': 'chris',
        'full_name': 'Chris Messina',
        'is_private': False,
        'profile_pic_url': 'https://scontent-amt2-1.cdninstagram.com/t51.2885-19/s150x150/23824744_1036957976446940_7940760494346862592_n.jpg',
        'profile_pic_id': '1654528278723030076_19',
        'friendship_status': {
            'following': False,
            'outgoing_request': False,
            'is_bestie': False
        },
        'is_verified': False,
        'has_anonymous_profile_picture': False,
        'is_unpublished': False,
        'is_favorite': False
    },
    'caption': None,
    'caption_is_edited': False,
    'like_count': 260,
    'has_liked': False,
    'top_likers': [],
    'comment_likes_enabled': True,
    'comment_threading_enabled': False,
    'has_more_comments': True,
    'max_num_visible_preview_comments': 2,
    'preview_comments': [],
    'comment_count': 18,
    'photo_of_you': False,
    'can_viewer_save': True,
    'organic_tracking_token': 'eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiYmVkOTQ3ODIzODlkNDE2Nzg2ZTExNDc5ZmExMTkyYTIxMjM0Iiwic2VydmVyX3Rva2VuIjoiMTUxNjIyMTI0MzYxMXwxMjM0fDE4MjY5NjAwNnw5NGQzMjMyMmUxYzgxM2U3MWJmYTZjYzkzZjgxMTgyYzZmMzFmMGUyZTA2ODFjZjI1YzA4YzBiZDFkMWQ3M2U5In0sInNpZ25hdHVyZSI6IiJ9'
}

TEST_TIMELINE_PHOTO_ITEM = {
    "media_or_ad": {
        'taken_at': 1281669687,
        'pk': 1234,
        'id': '1234_19',
        'device_timestamp': 1281669538,
        'media_type': 1,
        'code': 'TS',
        'client_cache_key': 'MTIzNA==.2',
        'filter_type': 0,
        'image_versions2': {
            'candidates': [
                {
                    'width': 612,
                    'height': 612,
                    'url': 'https://scontent-amt2-1.cdninstagram.com/vp/9ef94dbfea2b8cdb2ba5c9b45f1945fd/5AEFE328/t51.2885-15/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2'
                },
                {
                    'width': 240,
                    'height': 240,
                    'url': 'https://scontent-amt2-1.cdninstagram.com/vp/3f4554f2e45bf356951b2a437ab7d5b1/5ADE3631/t51.2885-15/s240x240/e15/11137637_1567371843535625_96536034_n.jpg?ig_cache_key=MTIzNA%3D%3D.2'}]
        },
        'original_width': 612,
        'original_height': 612,
        'lat': 37.3988349713,
        'lng': -122.027721405,
        'user': {
            'pk': 19,
            'username': 'chris',
            'full_name': 'Chris Messina',
            'is_private': False,
            'profile_pic_url': 'https://scontent-amt2-1.cdninstagram.com/t51.2885-19/s150x150/23824744_1036957976446940_7940760494346862592_n.jpg',
            'profile_pic_id': '1654528278723030076_19',
            'friendship_status': {
                'following': False,
                'outgoing_request': False,
                'is_bestie': False
            },
            'is_verified': False,
            'has_anonymous_profile_picture': False,
            'is_unpublished': False,
            'is_favorite': False
        },
        'caption': None,
        'caption_is_edited': False,
        'like_count': 260,
        'has_liked': False,
        'top_likers': [],
        'comment_likes_enabled': True,
        'comment_threading_enabled': False,
        'has_more_comments': True,
        'max_num_visible_preview_comments': 2,
        'preview_comments': [],
        'comment_count': 18,
        'photo_of_you': False,
        'can_viewer_save': True,
        'organic_tracking_token': 'eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiYmVkOTQ3ODIzODlkNDE2Nzg2ZTExNDc5ZmExMTkyYTIxMjM0Iiwic2VydmVyX3Rva2VuIjoiMTUxNjIyMTI0MzYxMXwxMjM0fDE4MjY5NjAwNnw5NGQzMjMyMmUxYzgxM2U3MWJmYTZjYzkzZjgxMTgyYzZmMzFmMGUyZTA2ODFjZjI1YzA4YzBiZDFkMWQ3M2U5In0sInNpZ25hdHVyZSI6IiJ9'
    }
}

TEST_USER_TAG_ITEM = {
    u'code': u'BRv8jaDFbsJ',
    u'can_viewer_save': True,
    u'usertags': {
        u'in': [
            {
                u'position': [0.8449074, 0.106481485],
                u'start_time_in_video_in_sec': None,
                u'user': {
                    u'username': u'iceprincezamani',
                    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/af79705613258faaa711b019f7ba7ac4/5CCEBDB3/t51.2885-19/s150x150/41606837_461088254381942_1625040398730657792_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
                    u'profile_pic_id': u'1869904537540938101_184565065',
                    u'full_name': u'Panshak Zamani :-)',
                    u'pk': 184565065,
                    u'is_verified': True,
                    u'is_private': False},
                u'duration_in_video_in_sec': None},
            {
                u'position': [0.6053240999999999, 0.9638888999999999],
                u'start_time_in_video_in_sec': None,
                u'user': {
                    u'username': u'xsproject',
                    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/ce16e991b80484564d2e9b8d6af70094/5CD13391/t51.2885-19/10570253_285800514958746_1157588212_a.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
                    u'full_name': u'XS PROJECT',
                    u'pk': 254488727,
                    u'is_verified': False,
                    u'is_private': False},
                u'duration_in_video_in_sec': None},
            {
                u'position': [0.23263889999999998, 0.7046296],
                u'start_time_in_video_in_sec': None, u'user': {
                    u'username': u'maxdevblock',
                    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/ff25ff3b684dea00e1918081c93c20e2/5CBA9941/t51.2885-19/s150x150/15876423_379687975701910_4035551486069964800_a.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
                    u'profile_pic_id': u'1419754651084223374_3998456661',
                    u'full_name': u'\U0001f13c\U0001f130\U0001f147\U0001f173\U0001f174\U0001f185\U0001f131\U0001f13b\U0001f13e\U0001f132\U0001f13a',
                    u'pk': 3998456661,
                    u'is_verified': False,
                    u'is_private': False},
                u'duration_in_video_in_sec': None}]},
    u'next_max_id': 17853655591136137,
    u'comment_likes_enabled': True,
    u'max_num_visible_preview_comments': 2,
    u'like_count': 52,
    u'image_versions2': {
        u'candidates': [
            {
                u'url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/aa092049c99e3ad1b6a4c862e347c261/5CD1FFFF/t51.2885-15/e35/17266207_160518517800958_3004377164544999424_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net&se=7&ig_cache_key=MTQ3MjY2MTkxOTExNDgzNjc0NQ%3D%3D.2',
                u'width': 1080,
                u'height': 1350},
            {
                u'url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/9ba27b08ab8a035f0151d6b9a4e2f87e/5CCD7CE4/t51.2885-15/e35/p240x240/17266207_160518517800958_3004377164544999424_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net&ig_cache_key=MTQ3MjY2MTkxOTExNDgzNjc0NQ%3D%3D.2',
                u'width': 240,
                u'height': 300}]},
    u'comment_threading_enabled': True,
    u'id': u'1472661919114836745_2308259673',
    u'preview_comments': [
        {
            u'status': u'Active',
            u'user_id': 4556478952,
            u'created_at_utc': 1489818661,
            u'created_at': 1489818661,
            u'bit_flags': 0,
            u'share_enabled': False,
            u'did_report_as_spam': False,
            u'user': {
                u'username': u'webdeveloper_world',
                u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/54a66674c1f1d5cd56b56c75e3d6ec1e/5CCA7BE8/t51.2885-19/s150x150/32233103_638476819831993_8407495672734941184_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
                u'profile_pic_id': u'1442122589220544777_4556478952',
                u'full_name': u'Webdeveloper World',
                u'pk': 4556478952,
                u'is_verified': False,
                u'is_private': False},
            u'content_type': u'comment',
            u'text': u'Brilliant! Best one so far!',
            u'media_id': 1472661919114836745,
            u'pk': 17862027220087150,
            u'type': 0}],
    u'client_cache_key': u'MTQ3MjY2MTkxOTExNDgzNjc0NQ==.2',
    u'device_timestamp': 1489774722152,
    u'comment_count': 3,
    u'media_type': 1,
    u'organic_tracking_token': u'eyJ2ZXJzaW9uIjo1LCJwYXlsb2FkIjp7ImlzX2FuYWx5dGljc190cmFja2VkIjp0cnVlLCJ1dWlkIjoiZjBiNjg4NGY1Zjg4NDk3ZDlkYmUwZDE2OWJhNmFlZGUxNDcyNjYxOTE5MTE0ODM2NzQ1Iiwic2VydmVyX3Rva2VuIjoiMTU0NzYzMDUzNjc1N3wxNDcyNjYxOTE5MTE0ODM2NzQ1fDM5OTg0NTY2NjF8YjY0YzFmNjRmZjM2NjJhMWY0NjgzZWFiY2ZmNzUwN2U4ZDM1OGU3ZDRmOWUyODBkNTljNWYwZjdlNDZiZGFkMyJ9LCJzaWduYXR1cmUiOiIifQ==',
    u'caption_is_edited': False,
    u'inline_composer_display_condition': u'impression_trigger',
    u'original_height': 1350,
    u'filter_type': 112,
    u'user': {
        u'username': u'monsieurwinner',
        u'reel_auto_archive': u'unset',
        u'has_anonymous_profile_picture': False,
        u'is_unpublished': False,
        u'is_favorite': False,
        u'friendship_status': {
            u'following': False,
            u'outgoing_request': False,
            u'is_bestie': False},
        u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/9f47c96e348c6eaaea7cf643ee626008/5CBCD374/t51.2885-19/s150x150/46915113_2288103657891057_3713770861216399360_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
        u'profile_pic_id': u'1942331692376923386_2308259673',
        u'full_name': u'MrWinner CEO Musiklibrary.com',
        u'pk': 2308259673,
        u'is_verified': False,
        u'is_private': False},
    u'pk': 1472661919114836745,
    u'has_liked': True,
    u'has_more_comments': True,
    u'can_view_more_preview_comments': True,
    u'photo_of_you': True,
    u'caption': {
        u'status': u'Active',
        u'user_id': 2308259673,
        u'created_at_utc': 1489775003,
        u'created_at': 1489775003,
        u'bit_flags': 0,
        u'share_enabled': False,
        u'did_report_as_spam': False,
        u'user': {
            u'username': u'monsieurwinner',
            u'reel_auto_archive': u'unset',
            u'has_anonymous_profile_picture': False,
            u'is_unpublished': False,
            u'is_favorite': False,
            u'friendship_status': {
                u'following': False,
                u'outgoing_request': False,
                u'is_bestie': False},
            u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/9f47c96e348c6eaaea7cf643ee626008/5CBCD374/t51.2885-19/s150x150/46915113_2288103657891057_3713770861216399360_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
            u'profile_pic_id': u'1942331692376923386_2308259673',
            u'full_name': u'MrWinner CEO Musiklibrary.com',
            u'pk': 2308259673,
            u'is_verified': False,
            u'is_private': False},
        u'content_type': u'comment',
        u'text': u"#Classicman Programmers are not always busy to the point that, they don't care for themselves. Programmers are #cute #handsome #smart #classicman. We love #program, #code #think #build #test #debugg #run.\n#developer #winner #php #java #webdeveloper #webdesigner #femalecoder #femaleprogrammer #blackAndSmart #blackandclean",
        u'media_id': 1472661919114836745,
        u'pk': 17874973114042066,
        u'type': 1},
    u'taken_at': 1489775002,
    u'original_width': 1080,
    u'can_viewer_reshare': True
}

TEST_MEDIA_LIKER = {
    u'full_name': u'Instagrambot Instabot',
    u'is_private': False,
    u'is_verified': False,
    u'pk': 9876543210,
    u'profile_pic_id': u'1234567890123456789_123456789',
    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
    u'username': u'instabot'
}

TEST_FOLLOWER_ITEM = {
    u'full_name': u'Follower Username',
    u'has_anonymous_profile_picture': False,
    u'is_private': False,
    u'is_verified': False,
    u'pk': 1234567890,
    u'profile_pic_id': u'1234567890123456789_1234567890',
    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
    u'reel_auto_archive': u'on',
    u'username': u'follower.username.'
}

TEST_FOLLOWING_ITEM = {
    u'full_name': u'Following Username',
    u'has_anonymous_profile_picture': False,
    u'is_favorite': False,
    u'is_private': False,
    u'is_verified': False,
    u'pk': 1234567890,
    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
    u'reel_auto_archive': u'unset',
    u'username': u'following.username'
}

TEST_COMMENT_LIKER_ITEM = {
    u'full_name': u'Instagrambot Instabot',
    u'is_private': False,
    u'is_verified': False,
    u'pk': 9876543210,
    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/1234567890abcdef1234567890abcdef/ABCDEF12/t00.0000-00/s150x150/12345678_1234567890123456_1234567890123456789_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
    u'username': u'instabot'
}

TEST_LOCATION_ITEM = {
    u'header_media': {},
    u'location': {
        u'address': u'',
        u'city': u'Location City Name',
        u'external_source': u'facebook_places',
        u'facebook_places_id': 123456789012345,
        u'lat': 1.2345,
        u'lng': 9.8765,
        u'name': u'City, State',
        u'pk': 123456789,
        u'short_name': u'City'},
    u'media_bundles': [],
    u'subtitle': u'',
    u'title': u'City, State'
}

TEST_MOST_RECENT_INVITER_ITEM = {
    u'full_name': u'Inbox User 1',
    u'has_anonymous_profile_picture': False,
    u'is_private': False,
    u'is_verified': False,
    u'pk': 12345678901,
    u'profile_pic_id': u'1952395551539983563_12345678901',
    u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/415ce786f601f6950cedaf19570ee0fd/5CC331E4/t51.2885-19/s150x150/49682915_326457101297245_1431392662795059200_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
    u'reel_auto_archive': u'on',
    u'username': u'inbox_user_1'
}

TEST_INBOX_THREAD_ITEM = {
    u'canonical': True,
    u'has_newer': False,
    u'has_older': True,
    u'inviter': {
        u'full_name': u'Inbox User 2',
        u'has_anonymous_profile_picture': False,
        u'is_private': False,
        u'is_verified': False,
        u'pk': 1234567890,
        u'profile_pic_id': u'1234567890123456789_1234567890',
        u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/e9c32c0f4591557f6beec9cb199a44b4/5CD22060/t51.2885-19/s150x150/12345678_293082444858234_5330756439625433088_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
        u'reel_auto_archive': u'on',
        u'username': u'inbox.user.2'},
    u'is_group': False,
    u'is_pin': False,
    u'is_spam': False,
    u'items': [{u'client_context': u'74a0440f-c5d1-441c-af16-d0e109fc6a3c',
                u'item_id': u'28544793378841687329460340129792000',
                u'item_type': u'text',
                u'text': u'AhHHHH cool',
                u'timestamp': 1547416349724500,
                u'user_id': 1098765432}],
    u'last_activity_at': 1547416349724500,
    u'last_permanent_item': {u'client_context': u'74a0440f-c5d1-441c-af16-d0e109fc6a3c',
                             u'item_id': u'28544793378841687329460340129792000',
                             u'item_type': u'text',
                             u'text': u'AhHHHH cool',
                             u'timestamp': 1547416349724500,
                             u'user_id': 1098765432},
    u'last_seen_at': {u'1098765432': {u'item_id': u'28544793378841687329460340129792000',
                                      u'timestamp': u'1547416349724500'}},
    u'left_users': [],
    u'mentions_muted': False,
    u'muted': False,
    u'named': False,
    u'newest_cursor': u'28544793378841687329460340129792000',
    u'oldest_cursor': u'28544793378841687329460340129792000',
    u'pending': False,
    u'pending_score': 1547416349724500,
    u'thread_id': u'340282366841710300949128210682725503544',
    u'thread_title': u'inbox.user.2',
    u'thread_type': u'private',
    u'thread_v2_id': u'17940844501242424',
    u'users': [{u'friendship_status': {u'blocking': False,
                                       u'following': True,
                                       u'incoming_request': False,
                                       u'is_bestie': False,
                                       u'is_private': False,
                                       u'outgoing_request': False},
                u'full_name': u'Inbox User 2',
                u'has_anonymous_profile_picture': False,
                u'is_directapp_installed': False,
                u'is_private': False,
                u'is_verified': False,
                u'pk': 1234567890,
                u'profile_pic_id': u'1234567890123456789_1234567890',
                u'profile_pic_url': u'https://instagram.fmxp5-1.fna.fbcdn.net/vp/e9c32c0f4591557f6beec9cb199a44b4/5CD22060/t51.2885-19/s150x150/12345678_293082444858234_5330756439625433088_n.jpg?_nc_ht=instagram.fmxp5-1.fna.fbcdn.net',
                u'reel_auto_archive': u'on',
                u'username': u'inbox.user.2'}],
    u'valued_request': False,
    u'vc_muted': False,
    u'viewer_id': 1098765432
}




import random
from collections import OrderedDict

from huepy import bold, green, orange


class file(object):
    def __init__(self, fname, verbose=True):
        self.fname = fname
        self.verbose = verbose
        open(self.fname, 'a').close()

    @property
    def list(self):
        with open(self.fname, 'r') as f:
            lines = [x.strip('\n') for x in f.readlines()]
            return [x for x in lines if x]

    @property
    def set(self):
        return set(self.list)

    def __iter__(self):
        for i in self.list:
            yield next(iter(i))

    def __len__(self):
        return len(self.list)

    def append(self, item, allow_duplicates=False):
        if self.verbose:
            msg = "Adding '{}' to `{}`.".format(item, self.fname)
            print(bold(green(msg)))

        if not allow_duplicates and str(item) in self.list:
            msg = "'{}' already in `{}`.".format(item, self.fname)
            print(bold(orange(msg)))
            return

        with open(self.fname, 'a') as f:
            f.write('{item}\n'.format(item=item))

    def remove(self, x):
        x = str(x)
        items = self.list
        if x in items:
            items.remove(x)
            msg = "Removing '{}' from `{}`.".format(x, self.fname)
            print(bold(green(msg)))
            self.save_list(items)

    def random(self):
        return random.choice(self.list)

    def remove_duplicates(self):
        return list(OrderedDict.fromkeys(self.list))

    def save_list(self, items):
        with open(self.fname, 'w') as f:
            for item in items:
                f.write('{item}\n'.format(item=item))



"""
    Filter functions for media and user lists.
"""


def filter_medias(self, media_items, filtration=True, quiet=False, is_comment=False):
    if filtration:
        if not quiet:
            self.logger.info("Received {} medias.".format(len(media_items)))
        if not is_comment:
            media_items = _filter_medias_not_liked(media_items)
            if self.max_likes_to_like:
                media_items = _filter_medias_nlikes(
                    media_items, self.max_likes_to_like, self.min_likes_to_like)
        else:
            media_items = _filter_medias_not_commented(self, media_items)
        if not quiet:
            msg = "After filtration {} medias left."
            self.logger.info(msg.format(len(media_items)))
    return _get_media_ids(media_items)


def _filter_medias_not_liked(media_items):
    not_liked_medias = []
    for media in media_items:
        if 'has_liked' in media and not media['has_liked']:
            not_liked_medias.append(media)
    return not_liked_medias


def _filter_medias_not_commented(self, media_items):
    not_commented_medias = []
    for media in media_items:
        if media.get('comment_count', 0) > 0 and media.get('comments'):
            my_comments = [comment for comment in media['comments']
                           if comment['user_id'] == self.user_id]
            if my_comments:
                continue
        not_commented_medias.append(media)
    return not_commented_medias


def _filter_medias_nlikes(media_items, max_likes_to_like, min_likes_to_like):
    filtered_medias = []
    for media in media_items:
        if 'like_count' in media:
            if media['like_count'] < max_likes_to_like and media['like_count'] > min_likes_to_like:
                filtered_medias.append(media)
    return filtered_medias


def _get_media_ids(media_items):
    result = []
    for media in media_items:
        if 'pk' in media:
            result.append(media['pk'])
    return result


def check_media(self, media_id):
    if self.api.media_info(media_id):
        medias = self.api.last_json["items"]

        if search_blacklist_hashtags_in_media(self, media_id):
            msg = 'Blacklist hashtag found in media, skipping!'
            self.console_print(msg, 'red')
            return False

        if self.filter_medias(medias, quiet=True):
            return check_user(self, self.get_media_owner(media_id))
        return False

    msg = 'Media ID error!'
    self.console_print(msg, 'red')
    return False


# Filter users

def search_stop_words_in_user(self, user_info):
    text = ''
    if 'biography' in user_info:
        text += user_info['biography'].lower()

    if 'username' in user_info:
        text += user_info['username'].lower()

    if 'full_name' in user_info:
        text += user_info['full_name'].lower()

    for stop_word in self.stop_words:
        if stop_word in text:
            return True

    return False


def search_blacklist_hashtags_in_media(self, media_id):
    media_info = self.get_media_info(media_id)
    text = media_info[0]['caption']['text'] if media_info[0]['caption'] else ''

    media_comments = self.get_media_comments(media_id)
    comments_number = min(6, len(media_comments))

    for i in range(0, comments_number):
        text += ''.join(media_comments[i]['text'])

    return any((h in text) for h in self.blacklist_hashtags)


def check_user(self, user_id, unfollowing=False):
    if not self.filter_users and not unfollowing:
        return True

    self.small_delay()
    user_id = self.convert_to_user_id(user_id)

    if not user_id:
        self.console_print('not user_id, skipping!', 'red')
        return False
    if user_id in self.whitelist:
        self.console_print('`user_id` in `self.whitelist`.', 'green')
        return True
    if user_id in self.blacklist:
        self.console_print('`user_id` in `self.blacklist`.', 'red')
        return False

    if user_id == str(self.user_id):
        self.console_print("`user_id` equals bot's `user_id`, skipping!", 'green')
        return False

    if user_id in self.following:
        if not unfollowing:
            # Log to Console
            self.console_print('Already following, skipping!', 'red')
        return False

    user_info = self.get_user_info(user_id)
    if not user_info:
        self.console_print('not `user_info`, skipping!', 'red')
        return False

    msg = 'USER_NAME: {username}, FOLLOWER: {followers}, FOLLOWING: {following}'
    follower_count = user_info["follower_count"]
    following_count = user_info["following_count"]
    self.console_print(msg.format(
        username=user_info["username"],
        followers=follower_count,
        following=following_count
    ))

    skipped = self.skipped_file
    followed = self.followed_file

    if not unfollowing:
        if self.filter_previously_followed and user_id in followed.list:
            self.console_print('info: account previously followed, skipping!', 'red')
            return False
    if "has_anonymous_profile_picture" in user_info and self.filter_users_without_profile_photo:
        if user_info["has_anonymous_profile_picture"]:
            self.console_print('info: account DOES NOT HAVE A PROFILE PHOTO, skipping! ', 'red')
            skipped.append(user_id)
            return False
    if "is_private" in user_info and self.filter_private_users:
        if user_info["is_private"]:
            self.console_print('info: account is PRIVATE, skipping! ', 'red')
            skipped.append(user_id)
            return False
    if "is_business" in user_info and self.filter_business_accounts:
        if user_info["is_business"]:
            self.console_print('info: is BUSINESS, skipping!', 'red')
            skipped.append(user_id)
            return False
    if "is_verified" in user_info and self.filter_verified_accounts:
        if user_info["is_verified"]:
            self.console_print('info: is VERIFIED, skipping !', 'red')
            skipped.append(user_id)
            return False

    if follower_count < self.min_followers_to_follow:
        msg = 'follower_count < bot.min_followers_to_follow, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False
    if follower_count > self.max_followers_to_follow:
        msg = 'follower_count > bot.max_followers_to_follow, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False
    if user_info["following_count"] < self.min_following_to_follow:
        msg = 'following_count < bot.min_following_to_follow, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False
    if user_info["following_count"] > self.max_following_to_follow:
        msg = 'following_count > bot.max_following_to_follow, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False
    try:
        if follower_count / following_count > self.max_followers_to_following_ratio:
            msg = 'follower_count / following_count > bot.max_followers_to_following_ratio, skipping!'
            self.console_print(msg, 'red')
            skipped.append(user_id)
            return False
        if following_count / follower_count > self.max_following_to_followers_ratio:
            msg = 'following_count / follower_count > bot.max_following_to_followers_ratio, skipping!'
            self.console_print(msg, 'red')
            skipped.append(user_id)
            return False
    except ZeroDivisionError:
        self.console_print('ZeroDivisionError: division by zero', 'red')
        return False

    if 'media_count' in user_info and user_info["media_count"] < self.min_media_count_to_follow:
        msg = 'media_count < bot.min_media_count_to_follow, BOT or INACTIVE, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False

    if search_stop_words_in_user(self, user_info):
        msg = '`bot.search_stop_words_in_user` found in user, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False

    return True


def check_not_bot(self, user_id):
    """ Filter bot from real users. """
    self.small_delay()
    user_id = self.convert_to_user_id(user_id)
    if not user_id:
        return False
    if user_id in self.whitelist:
        return True
    if user_id in self.blacklist:
        return False

    user_info = self.get_user_info(user_id)
    if not user_info:
        return True  # closed acc

    skipped = self.skipped_file
    if "following_count" in user_info and user_info["following_count"] > self.max_following_to_block:
        msg = 'following_count > bot.max_following_to_block, skipping!'
        self.console_print(msg, 'red')
        skipped.append(user_id)
        return False  # massfollower

    if search_stop_words_in_user(self, user_info):
        msg = '`bot.search_stop_words_in_user` found in user, skipping!'
        skipped.append(user_id)
        return False

    return True




import pytest
import responses

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from instabot.api.config import API_URL, SIG_KEY_VERSION

from .test_bot import TestBot
from .test_variables import (TEST_SEARCH_USERNAME_ITEM, TEST_USERNAME_INFO_ITEM, TEST_FOLLOWER_ITEM,
                             TEST_FOLLOWING_ITEM)


def reset_files(_bot):
    for x in _bot.followed_file.list:
        _bot.followed_file.remove(x)
    for x in _bot.unfollowed_file.list:
        _bot.unfollowed_file.remove(x)
    for x in _bot.skipped_file.list:
        _bot.skipped_file.remove(x)


class TestBotFilter(TestBot):

    @responses.activate
    @pytest.mark.parametrize('username', [TEST_SEARCH_USERNAME_ITEM['username'],
                                          TEST_SEARCH_USERNAME_ITEM['pk'],
                                          str(TEST_SEARCH_USERNAME_ITEM['pk'])])
    @patch('time.sleep', return_value=None)
    def test_follow(self, patched_time_sleep, username):
        follows_at_start = self.bot.total['follows']
        self.bot._following = [1]
        reset_files(self.bot)
        user_id = TEST_SEARCH_USERNAME_ITEM['pk']
        my_test_search_username_item = TEST_SEARCH_USERNAME_ITEM.copy()
        my_test_username_info_item = TEST_USERNAME_INFO_ITEM.copy()
        my_test_search_username_item['is_verified'] = False
        my_test_search_username_item['is_business'] = False
        my_test_search_username_item['is_private'] = False
        my_test_search_username_item['follower_count'] = 100
        my_test_search_username_item['following_count'] = 15
        my_test_search_username_item['media_count'] = self.bot.min_media_count_to_follow + 1
        my_test_search_username_item['has_anonymous_profile_picture'] = False
        my_test_username_info_item['pk'] = TEST_SEARCH_USERNAME_ITEM['pk']
        my_test_username_info_item['username'] = TEST_SEARCH_USERNAME_ITEM['username']
        my_test_username_info_item['is_verified'] = False
        my_test_username_info_item['is_business'] = False
        my_test_username_info_item['is_private'] = False
        my_test_username_info_item['follower_count'] = 100
        my_test_username_info_item['following_count'] = 15
        my_test_username_info_item['media_count'] = self.bot.min_media_count_to_follow + 1
        my_test_username_info_item['has_anonymous_profile_picture'] = False

        response_data = {
            'status': 'ok',
            'user': my_test_search_username_item
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=username
            ), status=200, json=response_data)

        response_data = {
            'status': 'ok',
            'user': my_test_username_info_item
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=user_id
            ), status=200, json=response_data)

        response_data = {'status': 'ok'}
        responses.add(
            responses.POST, '{api_url}friendships/create/{user_id}/'.format(
                api_url=API_URL, user_id=user_id
            ), json=response_data, status=200)

        assert self.bot.follow(username)
        assert self.bot.total['follows'] == follows_at_start + 1
        assert self.bot.followed_file.list[-1] == str(user_id)
        assert str(user_id) in self.bot.following

    @responses.activate
    @pytest.mark.parametrize('user_ids', [
        [str(TEST_SEARCH_USERNAME_ITEM['pk']),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 1),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 2),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 3)],
        [str(TEST_SEARCH_USERNAME_ITEM['pk']),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 4),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 5),
         str(TEST_SEARCH_USERNAME_ITEM['pk'] + 6)]]
    )
    @patch('time.sleep', return_value=None)
    def test_follow_users(self, patched_time_sleep, user_ids):
        self.bot._following = [1]
        reset_files(self.bot)
        follows_at_start = self.bot.total['follows']
        self.bot.followed_file.append(str(user_ids[1]))
        self.bot.unfollowed_file.append(str(user_ids[2]))
        self.bot.skipped_file.append(str(user_ids[3]))

        my_test_search_username_item = TEST_SEARCH_USERNAME_ITEM.copy()
        my_test_username_info_item = TEST_USERNAME_INFO_ITEM.copy()

        my_test_search_username_item['is_verified'] = False
        my_test_search_username_item['is_business'] = False
        my_test_search_username_item['is_private'] = False
        my_test_search_username_item['follower_count'] = 100
        my_test_search_username_item['following_count'] = 15
        my_test_search_username_item['media_count'] = self.bot.min_media_count_to_follow + 1
        my_test_search_username_item['has_anonymous_profile_picture'] = False
        my_test_username_info_item['username'] = TEST_SEARCH_USERNAME_ITEM['username']
        my_test_username_info_item['is_verified'] = False
        my_test_username_info_item['is_business'] = False
        my_test_username_info_item['is_private'] = False
        my_test_username_info_item['follower_count'] = 100
        my_test_username_info_item['following_count'] = 15
        my_test_username_info_item['media_count'] = self.bot.min_media_count_to_follow + 1
        my_test_username_info_item['has_anonymous_profile_picture'] = False

        for user_id in user_ids:
            my_test_search_username_item['pk'] = user_id
            my_test_username_info_item['pk'] = user_id

            response_data = {
                'status': 'ok',
                'user': my_test_search_username_item
            }
            responses.add(
                responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                    api_url=API_URL, username=user_id
                ), status=200, json=response_data)

            response_data = {
                'status': 'ok',
                'user': my_test_username_info_item
            }
            responses.add(
                responses.GET, '{api_url}users/{user_id}/info/'.format(
                    api_url=API_URL, user_id=user_id
                ), status=200, json=response_data)

            response_data = {'status': 'ok'}
            responses.add(
                responses.POST, '{api_url}friendships/create/{user_id}/'.format(
                    api_url=API_URL, user_id=user_id
                ), json=response_data, status=200)

        test_broken_items = [] == self.bot.follow_users(user_ids)
        test_follows = self.bot.total['follows'] == follows_at_start + 1
        test_following = self.bot.following == [1, user_ids[0]]
        test_followed = str(user_ids[0]) in self.bot.followed_file.list
        assert (test_broken_items and test_follows and test_followed and test_following)

    @responses.activate
    @pytest.mark.parametrize('username', [
        '1234567890', 1234567890
    ])
    @patch('time.sleep', return_value=None)
    def test_follow_followers(self, patched_time_sleep, username):
        self.blacklist = []
        my_test_search_username_item = TEST_SEARCH_USERNAME_ITEM.copy()
        my_test_username_info_item = TEST_USERNAME_INFO_ITEM.copy()
        my_test_follower_item = TEST_FOLLOWER_ITEM.copy()

        self.bot._following = []
        reset_files(self.bot)
        follows_at_start = self.bot.total['follows']

        response_data_1 = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=username
            ), status=200, json=response_data_1)

        response_data_2 = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=username
            ), status=200, json=response_data_2)

        results_3 = 5
        my_test_follower_items = [my_test_follower_item.copy() for _ in range(results_3)]
        my_test_search_username_items = [my_test_search_username_item.copy() for _ in range(results_3)]
        my_test_username_info_items = [my_test_username_info_item.copy() for _ in range(results_3)]

        for i, _ in enumerate(range(results_3)):
            my_test_follower_items[i]['pk'] = TEST_FOLLOWER_ITEM['pk'] + i
            my_test_follower_items[i]['username'] = "{}_{}".format(TEST_FOLLOWER_ITEM['username'], i)
        response_data_3 = {
            'status': 'ok',
            'big_list': False,
            'next_max_id': None,
            'sections': None,
            'users': my_test_follower_items
        }
        responses.add(
            responses.GET, "{api_url}friendships/{user_id}/followers/?rank_token={rank_token}".format(
                api_url=API_URL, user_id=username, rank_token=self.bot.api.rank_token
            ), json=response_data_3, status=200)

        for i, _ in enumerate(range(results_3)):
            my_test_search_username_items[i]['username'] = "{}_{}".format(TEST_FOLLOWER_ITEM['username'], i)
            my_test_search_username_items[i]['pk'] = TEST_FOLLOWER_ITEM['pk'] + i
            my_test_search_username_items[i]['is_verified'] = False
            my_test_search_username_items[i]['is_business'] = False
            my_test_search_username_items[i]['is_private'] = False
            my_test_search_username_items[i]['follower_count'] = 100
            my_test_search_username_items[i]['following_count'] = 15
            my_test_search_username_items[i]['media_count'] = self.bot.min_media_count_to_follow + 1
            my_test_search_username_items[i]['has_anonymous_profile_picture'] = False

            my_test_username_info_items[i]['username'] = "{}_{}".format(TEST_FOLLOWER_ITEM['username'], i)
            my_test_username_info_items[i]['pk'] = TEST_FOLLOWER_ITEM['pk'] + i
            my_test_username_info_items[i]['is_verified'] = False
            my_test_username_info_items[i]['is_business'] = False
            my_test_username_info_items[i]['is_private'] = False
            my_test_username_info_items[i]['follower_count'] = 100
            my_test_username_info_items[i]['following_count'] = 15
            my_test_username_info_items[i]['media_count'] = self.bot.min_media_count_to_follow + 1
            my_test_username_info_items[i]['has_anonymous_profile_picture'] = False

            response_data = {
                'status': 'ok',
                'user': my_test_search_username_items[i]
            }
            responses.add(
                responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                    api_url=API_URL, username=my_test_search_username_items[i]['username']
                ), status=200, json=response_data)

            response_data = {
                'status': 'ok',
                'user': my_test_username_info_items[i]
            }
            responses.add(
                responses.GET, '{api_url}users/{user_id}/info/'.format(
                    api_url=API_URL, user_id=my_test_username_info_items[i]['pk']
                ), status=200, json=response_data)

            response_data = {'status': 'ok'}
            responses.add(
                responses.POST, '{api_url}friendships/create/{user_id}/'.format(
                    api_url=API_URL, user_id=my_test_username_info_items[i]['pk']
                ), json=response_data, status=200)

        self.bot.follow_followers(username)

        test_follows = self.bot.total['follows'] == follows_at_start + results_3
        test_following = sorted(self.bot.following) == [str(my_test_username_info_items[i]['pk']) for i in range(results_3)]
        test_followed = sorted(self.bot.followed_file.list) == [str(my_test_username_info_items[i]['pk']) for i in range(results_3)]
        assert (test_follows and test_following and test_followed)

    @responses.activate
    @pytest.mark.parametrize('username', [
        '1234567890', 1234567890
    ])
    @patch('time.sleep', return_value=None)
    def test_follow_following(self, patched_time_sleep, username):
        self.blacklist = []
        my_test_search_username_item = TEST_SEARCH_USERNAME_ITEM.copy()
        my_test_username_info_item = TEST_USERNAME_INFO_ITEM.copy()
        my_test_following_item = TEST_FOLLOWING_ITEM.copy()

        self.bot._following = []
        reset_files(self.bot)
        follows_at_start = self.bot.total['follows']

        response_data_1 = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=username
            ), status=200, json=response_data_1)

        response_data_2 = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=username
            ), status=200, json=response_data_2)

        results_3 = 5
        my_test_following_items = [my_test_following_item.copy() for _ in range(results_3)]
        my_test_search_username_items = [my_test_search_username_item.copy() for _ in range(results_3)]
        my_test_username_info_items = [my_test_username_info_item.copy() for _ in range(results_3)]

        for i, _ in enumerate(range(results_3)):
            my_test_following_items[i]['pk'] = TEST_FOLLOWING_ITEM['pk'] + i
            my_test_following_items[i]['username'] = "{}_{}".format(TEST_FOLLOWING_ITEM['username'], i)
        response_data_3 = {
            'status': 'ok',
            'big_list': False,
            'next_max_id': None,
            'sections': None,
            'users': my_test_following_items
        }
        responses.add(
            responses.GET, "{api_url}friendships/{user_id}/following/?max_id={max_id}&ig_sig_key_version={sig_key}&rank_token={rank_token}".format(
                api_url=API_URL, user_id=username, rank_token=self.bot.api.rank_token, sig_key=SIG_KEY_VERSION, max_id=''
            ), json=response_data_3, status=200)

        for i, _ in enumerate(range(results_3)):
            my_test_search_username_items[i]['username'] = "{}_{}".format(TEST_FOLLOWING_ITEM['username'], i)
            my_test_search_username_items[i]['pk'] = TEST_FOLLOWING_ITEM['pk'] + i
            my_test_search_username_items[i]['is_verified'] = False
            my_test_search_username_items[i]['is_business'] = False
            my_test_search_username_items[i]['is_private'] = False
            my_test_search_username_items[i]['follower_count'] = 100
            my_test_search_username_items[i]['following_count'] = 15
            my_test_search_username_items[i]['media_count'] = self.bot.min_media_count_to_follow + 1
            my_test_search_username_items[i]['has_anonymous_profile_picture'] = False

            my_test_username_info_items[i]['username'] = "{}_{}".format(TEST_FOLLOWING_ITEM['username'], i)
            my_test_username_info_items[i]['pk'] = TEST_FOLLOWING_ITEM['pk'] + i
            my_test_username_info_items[i]['is_verified'] = False
            my_test_username_info_items[i]['is_business'] = False
            my_test_username_info_items[i]['is_private'] = False
            my_test_username_info_items[i]['follower_count'] = 100
            my_test_username_info_items[i]['following_count'] = 15
            my_test_username_info_items[i]['media_count'] = self.bot.min_media_count_to_follow + 1
            my_test_username_info_items[i]['has_anonymous_profile_picture'] = False

            response_data = {
                'status': 'ok',
                'user': my_test_search_username_items[i]
            }
            responses.add(
                responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                    api_url=API_URL, username=my_test_search_username_items[i]['username']
                ), status=200, json=response_data)

            response_data = {
                'status': 'ok',
                'user': my_test_username_info_items[i]
            }
            responses.add(
                responses.GET, '{api_url}users/{user_id}/info/'.format(
                    api_url=API_URL, user_id=my_test_username_info_items[i]['pk']
                ), status=200, json=response_data)

            response_data = {'status': 'ok'}
            responses.add(
                responses.POST, '{api_url}friendships/create/{user_id}/'.format(
                    api_url=API_URL, user_id=my_test_username_info_items[i]['pk']
                ), json=response_data, status=200)

        self.bot.follow_following(username)

        test_follows = self.bot.total['follows'] == follows_at_start + results_3
        test_following = sorted(self.bot.following) == [str(my_test_username_info_items[i]['pk']) for i in range(results_3)]
        test_followed = sorted(self.bot.followed_file.list) == [str(my_test_username_info_items[i]['pk']) for i in range(results_3)]
        assert (test_follows and test_following and test_followed)



"""
    All methods must return media_ids that can be
    passed into e.g. like() or comment() functions.
"""

from tqdm import tqdm


def get_media_owner(self, media_id):
    self.api.media_info(media_id)
    try:
        return str(self.api.last_json["items"][0]["user"]["pk"])
    except Exception as ex:
        self.logger.error("Error: get_media_owner(%s)\n%s", media_id, ex)
        return False


def get_user_tags_medias(self, user_id):
    self.api.get_user_tags(user_id)
    return [str(media['pk']) for media in self.api.last_json['items']]


def get_popular_medias(self):
    self.api.get_popular_feed()
    return [str(media['pk']) for media in self.api.last_json['items']]


def get_your_medias(self, as_dict=False):
    self.api.get_self_user_feed()
    if as_dict:
        return self.api.last_json["items"]
    return self.filter_medias(self.api.last_json["items"], False)


def get_archived_medias(self, as_dict=False):
    self.api.get_archive_feed()
    if as_dict:
        return self.api.last_json["items"]
    return self.filter_medias(self.api.last_json["items"], False)


def get_timeline_medias(self, filtration=True):
    if not self.api.get_timeline_feed():
        self.logger.warning("Error while getting timeline feed.")
        return []

    feed_items = [
        item["media_or_ad"]
        for item in self.api.last_json["feed_items"]
        if item.get("media_or_ad")
    ]
    return self.filter_medias(feed_items, filtration)


def get_user_medias(self, user_id, filtration=True, is_comment=False):
    user_id = self.convert_to_user_id(user_id)
    self.api.get_user_feed(user_id)
    if self.api.last_json["status"] == 'fail':
        self.logger.warning("This is a closed account.")
        return []
    return self.filter_medias(self.api.last_json["items"], filtration, is_comment=is_comment)


def get_total_user_medias(self, user_id):
    user_id = self.convert_to_user_id(user_id)
    medias = self.api.get_total_user_feed(user_id)
    if self.api.last_json["status"] == 'fail':
        self.logger.warning("This is a closed account.")
        return []
    return self.filter_medias(medias, filtration=False)


def get_last_user_medias(self, user_id, amount):
    user_id = self.convert_to_user_id(user_id)
    medias = self.api.get_last_user_feed(user_id, amount)
    if self.api.last_json["status"] == 'fail':
        self.logger.warning("This is a closed account.")
        return []
    return self.filter_medias(medias, filtration=False)


def get_user_likers(self, user_id, media_count=10):
    your_likers = set()
    media_items = self.get_user_medias(user_id, filtration=False)
    if not media_items:
        self.logger.warning("Can't get %s medias." % user_id)
        return []
    for media_id in tqdm(media_items[:media_count],
                         desc="Getting %s media likers" % user_id):
        media_likers = self.get_media_likers(media_id)
        your_likers |= set(media_likers)
    return list(your_likers)


def get_hashtag_medias(self, hashtag, filtration=True):
    if not self.api.get_hashtag_feed(hashtag):
        self.logger.warning("Error while getting hashtag feed.")
        return []
    return self.filter_medias(self.api.last_json["items"], filtration)


def get_total_hashtag_medias(self, hashtag, amount=100, filtration=False):
    medias = self.api.get_total_hashtag_feed(hashtag, amount)

    return self.filter_medias(medias, filtration=filtration)


def get_geotag_medias(self, geotag, filtration=True):
    # TODO: returns list of medias from geotag
    pass


def get_locations_from_coordinates(self, latitude, longitude):
    self.api.search_location(lat=latitude, lng=longitude)
    all_locations = self.api.last_json["items"]
    filtered_locations = []

    for location in all_locations:
        location_lat = location["location"]["lat"]
        location_lng = location["location"]["lng"]

        if int(location_lat) == int(latitude) and int(location_lng) == int(longitude):
            filtered_locations.append(location)

    return filtered_locations


def get_media_info(self, media_id):
    if isinstance(media_id, dict):
        return media_id
    self.api.media_info(media_id)
    if "items" not in self.api.last_json:
        self.logger.info("Media with %s not found." % media_id)
        return []
    return self.api.last_json["items"]


def get_timeline_users(self):
    if not self.api.get_timeline_feed():
        self.logger.warning("Error while getting timeline feed.")
        return []
    if 'items' in self.api.last_json:
        return [str(i['user']['pk']) for i in self.api.last_json['items'] if i.get('user')]
    elif 'feed_items' in self.api.last_json:
        return [str(i['media_or_ad']['user']['pk']) for i in self.api.last_json['feed_items'] if i.get('media_or_ad', {}).get('user')]
    self.logger.info("Users for timeline not found.")
    return []


def get_hashtag_users(self, hashtag):
    if not self.api.get_hashtag_feed(hashtag):
        self.logger.warning("Error while getting hashtag feed.")
        return []
    return [str(i['user']['pk']) for i in self.api.last_json['items']]


def get_geotag_users(self, geotag):
    # TODO: returns list user_ids who just posted on this geotag
    pass


def get_user_id_from_username(self, username):
    if username not in self._usernames:
        self.api.search_username(username)
        self.very_small_delay()
        if "user" in self.api.last_json:
            self._usernames[username] = str(self.api.last_json["user"]["pk"])
        else:
            return None
    return self._usernames[username]


def get_username_from_user_id(self, user_id):
    user_info = self.get_user_info(user_id)
    if user_info and "username" in user_info:
        return str(user_info["username"])
    return None  # Not found


def get_user_info(self, user_id, use_cache=True):
    user_id = self.convert_to_user_id(user_id)
    if not use_cache or user_id not in self._user_infos:
        self.api.get_username_info(user_id)
        last_json = self.api.last_json
        if last_json is None or 'user' not in last_json:
            return False
        user_info = last_json['user']
        self._user_infos[user_id] = user_info
    return self._user_infos[user_id]


def get_user_followers(self, user_id, nfollows):
    user_id = self.convert_to_user_id(user_id)
    followers = self.api.get_total_followers(user_id, nfollows)
    return [str(item['pk']) for item in followers][::-1] if followers else []


def get_user_following(self, user_id, nfollows=None):
    user_id = self.convert_to_user_id(user_id)
    following = self.api.get_total_followings(user_id, nfollows)
    return [str(item['pk']) for item in following][::-1] if following else []


def get_comment_likers(self, comment_id):
    self.api.get_comment_likers(comment_id)
    if "users" not in self.api.last_json:
        self.logger.info("Comment with %s not found." % comment_id)
        return []
    return list(map(lambda user: str(user['pk']), self.api.last_json["users"]))


def get_media_likers(self, media_id):
    self.api.get_media_likers(media_id)
    if "users" not in self.api.last_json:
        self.logger.info("Media with %s not found." % media_id)
        return []
    return list(map(lambda user: str(user['pk']), self.api.last_json["users"]))


def get_media_comments(self, media_id, only_text=False):
    self.api.get_media_comments(media_id)
    if 'comments' not in self.api.last_json:
        return []
    if only_text:
        return [str(item["text"]) for item in self.api.last_json['comments']]
    return self.api.last_json['comments']


def get_media_comments_all(self, media_id, only_text=False, count=False):
    has_more_comments = True
    max_id = ''
    comments = []

    while has_more_comments:
        self.api.get_media_comments(media_id, max_id=max_id)
        for comment in self.api.last_json['comments']:
            comments.append(comment)
        has_more_comments = self.api.last_json['has_more_comments']
        if count and len(comments) >= count:
            comments = comments[:count]
            has_more_comments = False
            self.logger.info("Getting comments stopped by count (%s)." % count)
        if has_more_comments:
            max_id = self.api.last_json['next_max_id']

    if only_text:
        return [str(item["text"]) for item in sorted(
            comments, key=lambda k: k['created_at_utc'], reverse=False)]
    return sorted(comments, key=lambda k: k['created_at_utc'], reverse=False)


def get_media_commenters(self, media_id):
    self.get_media_comments(media_id)
    if 'comments' not in self.api.last_json:
        return []
    return [str(item["user"]["pk"]) for item in self.api.last_json['comments']]


def search_users(self, query):
    self.api.search_users(query)
    if "users" not in self.api.last_json:
        self.logger.info("Users with %s not found." % query)
        return []
    return [str(user['pk']) for user in self.api.last_json['users']]


def get_comment(self):
    try:
        return self.comments_file.random().strip()
    except IndexError:
        return "Wow!"


def get_media_id_from_link(self, link):
    if 'instagram.com/p/' not in link:
        self.logger.error('Unexpected link')
        return False
    link = link.split('/')
    code = link[link.index('p') + 1]

    alphabet = {
        '-': 62, '1': 53, '0': 52, '3': 55, '2': 54, '5': 57, '4': 56,
        '7': 59, '6': 58, '9': 61, '8': 60, 'A': 0, 'C': 2, 'B': 1,
        'E': 4, 'D': 3, 'G': 6, 'F': 5, 'I': 8, 'H': 7, 'K': 10, 'J': 9,
        'M': 12, 'L': 11, 'O': 14, 'N': 13, 'Q': 16, 'P': 15, 'S': 18,
        'R': 17, 'U': 20, 'T': 19, 'W': 22, 'V': 21, 'Y': 24, 'X': 23,
        'Z': 25, '_': 63, 'a': 26, 'c': 28, 'b': 27, 'e': 30, 'd': 29,
        'g': 32, 'f': 31, 'i': 34, 'h': 33, 'k': 36, 'j': 35, 'm': 38,
        'l': 37, 'o': 40, 'n': 39, 'q': 42, 'p': 41, 's': 44, 'r': 43,
        'u': 46, 't': 45, 'w': 48, 'v': 47, 'y': 50, 'x': 49, 'z': 51,
    }

    result = 0
    for char in code:
        result = result * 64 + alphabet[char]
    return result


def get_link_from_media_id(self, media_id):
    alphabet = {
        '-': 62, '1': 53, '0': 52, '3': 55, '2': 54, '5': 57, '4': 56,
        '7': 59, '6': 58, '9': 61, '8': 60, 'A': 0, 'C': 2, 'B': 1,
        'E': 4, 'D': 3, 'G': 6, 'F': 5, 'I': 8, 'H': 7, 'K': 10, 'J': 9,
        'M': 12, 'L': 11, 'O': 14, 'N': 13, 'Q': 16, 'P': 15, 'S': 18,
        'R': 17, 'U': 20, 'T': 19, 'W': 22, 'V': 21, 'Y': 24, 'X': 23,
        'Z': 25, '_': 63, 'a': 26, 'c': 28, 'b': 27, 'e': 30, 'd': 29,
        'g': 32, 'f': 31, 'i': 34, 'h': 33, 'k': 36, 'j': 35, 'm': 38,
        'l': 37, 'o': 40, 'n': 39, 'q': 42, 'p': 41, 's': 44, 'r': 43,
        'u': 46, 't': 45, 'w': 48, 'v': 47, 'y': 50, 'x': 49, 'z': 51,
    }
    result = ''
    while media_id:
        media_id, char = media_id // 64, media_id % 64
        result += list(alphabet.keys())[list(alphabet.values()).index(char)]
    return 'https://instagram.com/p/' + result[::-1] + '/'


def get_messages(self):
    if self.api.getv2Inbox():
        return self.api.last_json
    else:
        self.logger.info("Messages were not found, something went wrong.")
        return None


def convert_to_user_id(self, x):
    x = str(x)
    if not x.isdigit():
        x = x.lstrip('@')
        x = self.get_user_id_from_username(x)
    # if type is not str than it is int so user_id passed
    return x



from tqdm import tqdm


def like(self, media_id, check_media=True):
    if not self.reached_limit('likes'):
        if self.blocked_actions['likes']:
            self.logger.warn('YOUR `LIKE` ACTION IS BLOCKED')
            if self.blocked_actions_protection:
                self.logger.warn('blocked_actions_protection ACTIVE. Skipping `like` action.')
                return False
        self.delay('like')
        if check_media and not self.check_media(media_id):
            return False
        _r = self.api.like(media_id)
        if _r == 'feedback_required':
            self.logger.error("`Like` action has been BLOCKED...!!!")
            self.blocked_actions['likes'] = True
            return False
        if _r:
            self.logger.info("Liked media %d." % media_id)
            self.total['likes'] += 1
            return True
    else:
        self.logger.info("Out of likes for today.")
    return False


def like_comment(self, comment_id):
    if not self.reached_limit('likes'):
        if self.blocked_actions['likes']:
            self.logger.warn('YOUR `LIKE` ACTION IS BLOCKED')
            if self.blocked_actions_protection:
                from datetime import timedelta
                next_reset = (self.start_time.date() + timedelta(days=1)).strftime("%Y-%m-%d %H:%M:%S")
                self.logger.warn('blocked_actions_protection ACTIVE. Skipping `like` action till, at least, {}.'.format(next_reset))
                return False
        self.delay('like')
        _r = self.api.like_comment(comment_id)
        if _r == 'feedback_required':
            self.logger.error("`Like` action has been BLOCKED...!!!")
            self.blocked_actions['likes'] = True
            return False
        if _r:
            self.logger.info("Liked comment {}.".format(comment_id))
            self.total['likes'] += 1
            return True
    else:
        self.logger.info("Out of likes for today.")
    return False


def like_media_comments(self, media_id):
    broken_items = []
    media_comments = self.get_media_comments(media_id)
    self.logger.info('Found {} comments'.format(len(media_comments)))
    comment_ids = [item["pk"] for item in media_comments if not item.get('has_liked_comment') or not item["has_liked_comment"]]

    if not comment_ids:
        self.logger.info("None comments received: comments not found or comments have been filtered.")
        return broken_items

    self.logger.info("Going to like %d comments." % (len(comment_ids)))

    for comment in tqdm(comment_ids):
        if not self.like_comment(comment):
            self.error_delay()
            broken_items = comment_ids[comment_ids.index(comment):]
    self.logger.info("DONE: Liked {count} comments.".format(
        count=len(comment_ids) - len(broken_items)
    ))
    return broken_items


def like_medias(self, medias, check_media=True):
    broken_items = []
    if not medias:
        self.logger.info("Nothing to like.")
        return broken_items
    self.logger.info("Going to like %d medias." % (len(medias)))
    for media in tqdm(medias):
        if not self.like(media, check_media):
            self.error_delay()
            broken_items.append(media)
    self.logger.info("DONE: Total liked %d medias." % self.total['likes'])
    return broken_items


def like_timeline(self, amount=None):
    self.logger.info("Liking timeline feed:")
    medias = self.get_timeline_medias()[:amount]
    return self.like_medias(medias, check_media=False)


def like_user(self, user_id, amount=None, filtration=True):
    """ Likes last user_id's medias """
    if filtration:
        if not self.check_user(user_id):
            return False
    self.logger.info("Liking user_%s's feed:" % user_id)
    user_id = self.convert_to_user_id(user_id)
    medias = self.get_user_medias(user_id, filtration=filtration)
    if not medias:
        self.logger.info(
            "None medias received: account is closed or medias have been filtered.")
        return False
    return self.like_medias(medias[:amount])


def like_users(self, user_ids, nlikes=None, filtration=True):
    for user_id in user_ids:
        if self.reached_limit('likes'):
            self.logger.info("Out of likes for today.")
            return
        self.like_user(user_id, amount=nlikes, filtration=filtration)


def like_hashtag(self, hashtag, amount=None):
    """ Likes last medias from hashtag """
    self.logger.info("Going to like media with hashtag #%s." % hashtag)
    medias = self.get_total_hashtag_medias(hashtag, amount)
    return self.like_medias(medias)


def like_geotag(self, geotag, amount=None):
    # TODO: like medias by geotag
    pass


def like_followers(self, user_id, nlikes=None, nfollows=None):
    self.logger.info("Like followers of: %s." % user_id)
    if self.reached_limit('likes'):
        self.logger.info("Out of likes for today.")
        return
    if not user_id:
        self.logger.info("User not found.")
        return
    follower_ids = self.get_user_followers(user_id, nfollows)
    if not follower_ids:
        self.logger.info("%s not found / closed / has no followers." % user_id)
    else:
        self.like_users(follower_ids[:nfollows], nlikes)


def like_following(self, user_id, nlikes=None, nfollows=None):
    self.logger.info("Like following of: %s." % user_id)
    if self.reached_limit('likes'):
        self.logger.info("Out of likes for today.")
        return
    if not user_id:
        self.logger.info("User not found.")
        return
    following_ids = self.get_user_following(user_id, nfollows)
    if not following_ids:
        self.logger.info("%s not found / closed / has no following." % user_id)
    else:
        self.like_users(following_ids, nlikes)
